/*****************************************************/
/*          This java file is a part of the          */
/*                                                   */
/*           -  Plouf's Java IRC Client  -           */
/*                                                   */
/*   Copyright (C)  2002 - 2003 Philippe Detournay   */
/*                                                   */
/*         All contacts : theplouf@yahoo.com         */
/*                                                   */
/*  PJIRC is free software; you can redistribute     */
/*  it and/or modify it under the terms of the GNU   */
/*  General Public License as published by the       */
/*  Free Software Foundation; version 2 or later of  */
/*  the License.                                     */
/*                                                   */
/*  PJIRC is distributed in the hope that it will    */
/*  be useful, but WITHOUT ANY WARRANTY; without     */
/*  even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  General Public License for more details.         */
/*                                                   */
/*  You should have received a copy of the GNU       */
/*  General Public License along with PJIRC; if      */
/*  not, write to the Free Software Foundation,      */
/*  Inc., 59 Temple Place, Suite 330, Boston,        */
/*  MA  02111-1307  USA                              */
/*                                                   */
/*****************************************************/

package irc;

import java.lang.reflect.*;
import java.util.*;

class DispatchThread extends Thread
{
  private Object _target;
  private String _method;
  private Object[] _params;
  private Object _lock;

  public DispatchThread(Object target,String method,Object[] params,Object lock)
  {
    _target=target;
    _method=method;
    _params=params;
    _lock=lock;
  }

  public void run()
  {
    if(_lock!=null)
    {
      try
      {
        synchronized(_lock)
        {
          _lock.wait();
        }
      }
      catch(InterruptedException ex)
      {
        return;
      }
    }
    EventDispatcher.dispatchEvent(_target,_method,_params);
  }
}

/**
 * Event dispatcher, using reflection.
 */
public class EventDispatcher
{
  private static Hashtable _cache=new Hashtable();

  private static boolean match(Class[] t1,Class[] t2)
  {
    if(t1.length!=t2.length) return false;
    for(int i=0;i<t1.length;i++)
    {
      if(t2[i]!=null)
        if(!t1[i].isAssignableFrom(t2[i])) return false;
    }
    return true;
  }

  /**
   * Clear the internal EventDispatcher method call.
   */
  public static void clearCache()
  {
    _cache.clear();
  }

  /**
   * Dispatch the given event in a new thread. Event return value is discarded.
   * @param target event target.
   * @param method event method name.
   * @param params event parameters.
   * @return the started thread in wich the event is called.
   */
  public static Thread dispatchEventAsync(Object target,String method,Object[] params)
  {
    return dispatchEventAsync(target,method,params,null);
  }

  /**
   * Dispatch the given event in a new thread. Event return value is discarded.
   * @param target event target.
   * @param method event method name.
   * @param params event parameters.
   * @param lock if not null, the newly created thread will wait on this lock before
   * attempting to execute the event. If the thread is interrupted while waiting, the
   * event will not be executed.
   * @return the started thread in wich the event is called.
   */
  public static Thread dispatchEventAsync(Object target,String method,Object[] params,Object lock)
  {
    Thread res=new DispatchThread(target,method,params,lock);
    res.start();
    return res;
  }


  /**
   * Dispatch a new event to the given target.
   * @param target target event listener.
   * @param method method name to call.
   * @param params parameters to pass to the called method.
   * @return method result.
   */
  public static Object dispatchEvent(Object target,String method,Object[] params)
  {
    try
    {
      Class c=target.getClass();
      Method m[]=(Method[])_cache.get(c);
      if(m==null)
      {
        m=c.getMethods();
        _cache.put(c,m);
      }
      Class types[]=new Class[params.length];
      for(int i=0;i<params.length;i++)
      {
        if(params[i]!=null)
          types[i]=params[i].getClass();
        else
          types[i]=null;
      }
      for(int i=0;i<m.length;i++)
      {
        if(m[i].getName().equals(method))
        {
          if(match(m[i].getParameterTypes(),types)) return m[i].invoke(target,params);
        }
      }
      throw new NoSuchMethodException(method);
    }
    catch(InvocationTargetException ex)
    {
      System.err.println("internal error");
      System.err.println("please submit a bug report to plouf@pjirc.com with the following information :");
      System.err.println("InvocationTargetException");
      System.err.println("Target exception trace :");
      if(ex.getTargetException()!=null)
        ex.getTargetException().printStackTrace();
      else
        System.err.println("Target exception is null");
      return null;

    }
    catch(Throwable ex)
    {
      System.err.println("internal error");
      System.err.println("please submit a bug report to plouf@pjirc.com with the following information :");
      ex.printStackTrace();
      return null;
    }
  }
}
