/*****************************************************/
/*          This java file is a part of the          */
/*                                                   */
/*           -  Plouf's Java IRC Client  -           */
/*                                                   */
/*   Copyright (C)  2002 - 2003 Philippe Detournay   */
/*                                                   */
/*         All contacts : theplouf@yahoo.com         */
/*                                                   */
/*  PJIRC is free software; you can redistribute     */
/*  it and/or modify it under the terms of the GNU   */
/*  General Public License as published by the       */
/*  Free Software Foundation; version 2 or later of  */
/*  the License.                                     */
/*                                                   */
/*  PJIRC is distributed in the hope that it will    */
/*  be useful, but WITHOUT ANY WARRANTY; without     */
/*  even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  General Public License for more details.         */
/*                                                   */
/*  You should have received a copy of the GNU       */
/*  General Public License along with PJIRC; if      */
/*  not, write to the Free Software Foundation,      */
/*  Inc., 59 Temple Place, Suite 330, Boston,        */
/*  MA  02111-1307  USA                              */
/*                                                   */
/*****************************************************/

package irc;

import java.awt.*;
import java.util.*;

import irc.security.*;

class NullItem
{
}

class MouseEventConfiguration
{
  public MouseEventConfiguration(int m,int c)
  {
    Mask=m;
    ClickCount=c;
  }

  public int Mask;
  public int ClickCount;
}

/**
 * Global IRC configuration.
 */
public class IRCConfiguration
{
  public static final int TILING_CENTER=0;
  public static final int TILING_STRETCH=1;
  public static final int TILING_TILE=2;

  private NullItem NULL_ITEM=new NullItem();

  private TextProvider _textProvider;
  private IRCColorModel _colorModel;
	private ImageLoader _loader;
  private URLHandler _handler;
	private FileHandler _file;
  private AudioConfiguration _audioConfig;

  private Vector _highLightWords;
	private SmileyTable _table;
  private ListHandler _mayJoinList;
  private ListHandler _mayLeaveList;
  private RuleList _backgroundImageRules;
  private RuleList _backgroundTilingRules;
  private RuleList _colorsRules;
  private RuleList _dockingRules;
  private ListHandler _mayCommandList;
  private Hashtable _htable;
  private Vector _nickMenuVector;
  private Hashtable _mouseConfig;
  private SecurityProvider _provider;


  /**
   * Create a new IRCConfiguration.
   * @param text text provider to use.
   * @param model color model to use.
   * @param handler URL handler to use.
   * @param loader image loader to use.
   * @param sound sound handler to use.
   * @param file file handler to use.
   */
  public IRCConfiguration(TextProvider text,IRCColorModel model,URLHandler handler,ImageLoader loader,SoundHandler sound,FileHandler file)
  {
    _provider=new SecurityProvider();

    _htable=new Hashtable();
    _mouseConfig=new Hashtable();

    _backgroundImageRules=new RuleList();
    _backgroundTilingRules=new RuleList();
    _backgroundTilingRules.setDefaultValue(new Integer(0));
    _dockingRules=new RuleList();
    _dockingRules.setDefaultValue(new Boolean(false));

    _colorsRules=new RuleList();
    Color[] def=new Color[16];
    loadDefaultColors(def);
    _colorsRules.setDefaultValue(def);

	  _nickMenuVector=new Vector();
    _highLightWords=new Vector();

		_audioConfig=new AudioConfiguration(sound);
    _table=new SmileyTable();

    _file=file;
    _loader=loader;
    _textProvider=text;
    _colorModel=model;
    _handler=handler;
  }

  /**
   * Get the high version number.
   * @return the high version number.
   */
  public int getVersionHigh()
  {
    return 1;
  }

  /**
   * Get the middle version number.
   * @return the middle version number.
   */
  public int getVersionMed()
  {
    return 9;
  }

  /**
   * Get the low version number.
   * @return the low version number.
   */
  public int getVersionLow()
  {
    return 0;
  }

  /**
   * Get the version modifiers.
   * @return version modifiers.
   */
  public String getVersionModifiers()
  {
    return "";
  }

  /**
   * Get version number as a string.
   * @return high.med.lowmod version number.
   */
  public String getVersion()
  {
    return getVersionHigh()+"."+getVersionMed()+"."+getVersionLow()+getVersionModifiers();
  }

  /**
   * Get the security provider.
   * @return the security provider.
   */
  public SecurityProvider getSecurityProvider()
  {
    return _provider;
  }

  /**
   * Configure the given mouse event name.
   * @param eventName event name.
   * @param button mouse button index.
   * @param count mouse click count.
   */
  public void setMouseConfiguration(String eventName,int button,int count)
  {
    int mask;
    switch(button)
    {
      case 1:mask=java.awt.event.MouseEvent.BUTTON1_MASK;break;
      case 2:mask=java.awt.event.MouseEvent.BUTTON2_MASK;break;
      case 3:mask=java.awt.event.MouseEvent.BUTTON3_MASK;break;
      default:mask=java.awt.event.MouseEvent.BUTTON1_MASK;break;
    }
    _mouseConfig.put(eventName.toLowerCase(),new MouseEventConfiguration(mask,count));
  }

  /**
   * Check whether the given event name and the given mouse event match.
   * @param eventName mouse event name.
   * @param event mouse event.
   * @return true if events match, false otherwise.
   */
  public boolean matchMouseConfiguration(String eventName,java.awt.event.MouseEvent event)
  {
    MouseEventConfiguration config=(MouseEventConfiguration)_mouseConfig.get(eventName.toLowerCase());
    if(config==null) throw new Error(eventName+" : unknown mouse event name");
    if(config.ClickCount!=event.getClickCount()) return false;
    return (event.getModifiers() & config.Mask)!=0;
  }

  /**
   * Set the given property to the given value. This value may be null.
   * @param key property name.
   * @param obj property value.
   */
  public void set(String key,Object obj)
  {
    if(obj==null) obj=NULL_ITEM;
    _htable.put(key.toLowerCase(),obj);
  }

  /**
   * Set the given property to the given int value.
   * @param key property name.
   * @param val property value.
   */
  public void set(String key,int val)
  {
    set(key,new Integer(val));
  }

  /**
   * Set the given property to the given boolean value.
   * @param key property name.
   * @param val property value.
   */
  public void set(String key,boolean val)
  {
    set(key,new Boolean(val));
  }

  /**
   * Get the given property value.
   * @param key property name.
   * @return the property value.
   * @throws RuntimeException if the property is unknown.
   */
  public Object get(String key)
  {
    Object v=_htable.get(key.toLowerCase());
    if(v==null) throw new RuntimeException("Unknown configuration property "+key);
    if(v==NULL_ITEM) v=null;
    return v;
  }

  /**
   * Get the given property value as an int value.
   * @param key property name.
   * @return the property value.
   * @throws RuntimeException if the property is unknown.
   */
  public int getI(String key)
  {
    Integer i=(Integer)get(key);
    return i.intValue();
  }

  /**
   * Get the given property value as a boolean value.
   * @param key property name.
   * @return the property value.
   * @throws RuntimeException if the property is unknown.
   */
  public boolean getB(String key)
  {
    Boolean b=(Boolean)get(key);
    return b.booleanValue();
  }

  /**
   * Get the given property value as String value.
   * @param key property name.
   * @return the property value.
   * @throws RuntimeException if the property is unknown.
   */
  public String getS(String key)
  {
    return (String)get(key);
  }

  /**
   * Get the nick menu configuration vector.
   * @return the nick menu vector.
   */
  public Vector getNickMenuVector()
  {
    return _nickMenuVector;
  }


  /**
   * Set the docking policy associated with the given source type and name.
   * @param type source type.
   * @param name source name.
   * @param action true if source should be undocked, false otherwise.
   */
  public void setDockingPolicy(String type,String name,boolean action)
  {
    _dockingRules.addRule(new String[] {type,name},new Boolean(action));
  }

  /**
   * Get the docking policy for the given source type and name.
   * @param type source type.
   * @param name source name.
   * @return true if the source should be undocked, false otherwise.
   */
  public boolean getDockingPolicy(String type,String name)
  {
    return ((Boolean)(_dockingRules.findValue(new String[] {type,name}))).booleanValue();
  }

  /**
   * Get the background image associated with the given source type and name.
   * @param type source type.
   * @param name source name.
   * @return background image, or null if no image is to be displayed.
   */
  public Image getBackgroundImage(String type,String name)
  {
    if(!getB("BackgroundImage")) return null;
    return (Image)_backgroundImageRules.findValue(new String[] {type,name});
  }

  /**
   * Get the background image tiling associated with the given source type and name.
   * @param type source type.
   * @param name source name.
   * @return tiling mode for background image.
   */
  public int getBackgroundTiling(String type,String name)
  {
    return ((Integer)_backgroundTilingRules.findValue(new String[] {type,name})).intValue();
  }

  /**
   * Set the background image to be used for the given source type and name.
   * @param type source type.
   * @param name source name.
   * @param image image name.
   */
  public void setBackgroundImage(String type,String name,String image)
  {
    _backgroundImageRules.addRule(new String[] {type,name},_loader.getImage(image));
  }

  /**
   * Set the background image tiling to be used for the given source type and name.
   * @param type source type.
   * @param name source name.
   * @param tiling tiling mode to be used.
   */
  public void setBackgroundTiling(String type,String name,int tiling)
  {
    _backgroundTilingRules.addRule(new String[] {type,name},new Integer(tiling));
  }

  /**
   * Test wether the given channel may be left.
   * @param channel channel to be left.
   * @return true if channel may be left, false otherwise.
   */
  public boolean mayLeave(String channel)
  {
    return _mayLeaveList.inList(channel);
  }

  /**
   * Test wether the given channel may be joined.
   * @param channel channel to be joined.
   * @return true if channel may be joined, false otherwise.
   */
  public boolean mayJoin(String channel)
  {
    return _mayJoinList.inList(channel);
  }

  /**
   * Set the "may join" list.
   * @param list join list.
   */
  public void setJoinList(String list)
  {
    _mayJoinList=new ListHandler(list);
  }

  /**
   * Set the "may leave" list.
   * @param list leave list.
   */
  public void setLeaveList(String list)
  {
    _mayLeaveList=new ListHandler(list);
  }

  /**
   * Set the authorized command list.
   * @param list authorized command list.
   */
  public void setCommandList(String list)
  {
    _mayCommandList=new ListHandler(list);
  }

  /**
   * Check wether the given command may be executed by the user.
   * @param list command to test, not prefixed by /.
   * @return true if command may be executed, false otherwise.
   */
  public boolean mayCommand(String cmd)
  {
    if(cmd.startsWith("/")) cmd=cmd.substring(1);
    return _mayCommandList.inList(cmd);
  }

  /**
   * Format a string given asl info and current settings.
   * @param info user whois information.
   * @return formatted asl information. An empty string shouldn't be displayed.
   */
  public String formatASL(String info)
  {
    String noprefix=getS("NoASLDisplayPrefix");
    if((noprefix.length()>0) && (info.startsWith(noprefix))) return "";
    String orig=info;
    int pos=info.indexOf(' ');
    if(pos<0) return orig;

    String age=info.substring(0,pos).trim();
    info=info.substring(pos+1).trim();
    pos=info.indexOf(' ');
    if(pos<0) return orig;
    String gender=info.substring(0,pos).trim().toLowerCase();
    String location=info.substring(pos+1).trim();

    int text;
    if(gender.equals(getS("ASLMale").toLowerCase())) text=TextProvider.ASL_MALE;
    else if(gender.equals(getS("ASLFemale").toLowerCase())) text=TextProvider.ASL_FEMALE;
    else if(gender.equals(getS("ASLUnknown").toLowerCase())) text=TextProvider.ASL_UNKNOWN;
    else return orig;

    return getText(text,age,location);
  }

  /**
   * Find the correct ASL color for the given info.
   * @param info user whois information.
   * @param male male color.
   * @param femeale femeale color.
   * @param undef undefined gender color.
   * @return asl color.
   */
  public Color getASLColor(String info,Color male,Color femeale,Color undef)
  {
    int pos=info.indexOf(' ');
    if(pos<0) return undef;
    info=info.substring(pos).trim();
    pos=info.indexOf(' ');
    if(pos<0) return undef;
    info=info.substring(0,pos).trim().toLowerCase();
    if(info.equals(getS("ASLMale").toLowerCase())) return male;
    if(info.equals(getS("ASLFemale").toLowerCase())) return femeale;
    return undef;
  }

  /**
   * Get the audio configuration.
   * @return the audio configuration.
   */
  public AudioConfiguration getAudioConfiguration()
	{
	  return _audioConfig;
	}

  /**
   * Get the default color context.
   * @return the default color context.
   */
  public ColorContext getDefaultColorContext()
	{
	  return getColorContext("","");
	}

  /**
   * Get the color context associated with the given source type and name.
   * @param type source name.
   * @param name source name.
   * @return associated color context.
   */
  public ColorContext getColorContext(String type,String name)
	{
	  ColorContext ctx=new ColorContext();
		ctx.type=type;
		ctx.name=name;
	  return ctx;
	}

  /**
   * Load the given color array with the default source colors. The array
   * must be of length 16.
   * @param cols color array to be filled.
   */
	public void loadDefaultColors(Color[] cols)
	{
    cols[0]=new Color(0xFFFFFF);
    cols[1]=new Color(0x000000);
    cols[2]=new Color(0x00007F);
    cols[3]=new Color(0x009300);
    cols[4]=new Color(0xFF0000);
    cols[5]=new Color(0x7F0000);
    cols[6]=new Color(0x9C009C);
    cols[7]=new Color(0xFC7F00);
    cols[8]=new Color(0xFFFF00);
    cols[9]=new Color(0x00FC00);
    cols[10]=new Color(0x009393);
    cols[11]=new Color(0x00FFFF);
    cols[12]=new Color(0x0000FC);
    cols[13]=new Color(0xFF00FF);
    cols[14]=new Color(0x7F7F7F);
    cols[15]=new Color(0xD2D2D2);
	}

  /**
   * Set the source colors.
   * @param type source type.
   * @param name source name.
   * @param c color array.
   */
  public void setSourceColor(String type,String name,Color c[])
	{
    _colorsRules.addRule(new String[] {type,name},c);
	}

	/**
	 * Get the colors associated with the given color context.
	 * @param context the context to get colors from.
	 * @return color array for the given context.
	 */
  public Color[] getStyleColors(ColorContext context)
	{
		return (Color[])_colorsRules.findValue(new String[] {context.type,context.name});
	}

	/**
	 * Add a smiley in the smiley table.
	 * @param match the matching text to replace.
	 * @param file image file name.
	 */
  public void addSmiley(String match,String file)
	{
	  _table.addSmiley(match,_loader.getImage(file));
	}

  /**
   * Get the image loader.
   * @return the image loader.
   */
  public ImageLoader getImageLoader()
	{
	  return _loader;
	}

  /**
   * Get the smileys table.
   * @return the smiley table.
   */
  public SmileyTable getSmileyTable()
	{
	  return _table;
	}

  /**
   * Return wether asl should be processed, because asl field is enabled or floating
   * asl is enabled.
   * @return master asl status.
   */
  public boolean getASLMaster()
  {
    return getB("ASL") | getB("FloatingASL");
  }

  /**
   * Set highlight configuration.
   * @param color hightlight color.
   * @param nick true if nick highlight enabled.
   * @param words hightlight words set.
   */
  public void setHighLightConfig(int color,boolean nick,Vector words)
  {
    set("HighLightColor",color);
    set("HighLightNick",nick);
    _highLightWords=new Vector();
    for(int i=0;i<words.size();i++) _highLightWords.insertElementAt(words.elementAt(i),_highLightWords.size());
  }

  /**
   * Get nick highlight flag.
   * @return nick highlight flag.
   */
  public boolean highLightNick()
  {
    return getB("HighLightNick") && getB("HighLight");
  }

  /**
   * Get highlight words.
   * @return enumeration of String.
   */
  public Enumeration getHighLightWords()
  {
    if(!getB("HighLight")) return new Vector().elements();
    return _highLightWords.elements();
  }

  /**
   * Get URLHandler.
   * @return URLHandler.
   */
  public URLHandler getURLHandler()
  {
    return _handler;
  }

  /**
   * Get text provider.
   * @return text provider.
   */
  public TextProvider getTextProvider()
  {
    return _textProvider;
  }

  /**
   * Get IRC color model.
   * @return color model.
   */
  public IRCColorModel getIRCColorModel()
  {
    return _colorModel;
  }

  /**
   * Get formatted text associated with the given text code, with no parameter.
   * @param code text code.
   * @return formatted text.
   */
  public String getText(int code)
  {
    return _textProvider.getString(code);
  }

  /**
   * Get formatted text associated with the given text code, with one parameter.
   * @param code text code.
   * @param p1 first parameter.
   * @return formatted text.
   */
  public String getText(int code,String p1)
  {
    return _textProvider.getString(code,p1);
  }

  /**
   * Get formatted text associated with the given text code, with two parameters.
   * @param code text code.
   * @param p1 first parameter.
   * @param p2 second parameter.
   * @return formatted text.
   */
  public String getText(int code,String p1,String p2)
  {
    return _textProvider.getString(code,p1,p2);
  }

  /**
   * Get formatted text associated with the given text code, with three parameters.
   * @param code text code.
   * @param p1 first parameter.
   * @param p2 second parameter.
   * @param p3 third parameter.
   * @return formatted text.
   */
  public String getText(int code,String p1,String p2,String p3)
  {
    return _textProvider.getString(code,p1,p2,p3);
  }

  /**
   * Get the i'th color of the color model.
   * @param i index.
   * @return color at index i.
   */
  public Color getColor(int i)
  {
    return _colorModel.getColor(i);
  }
}

