/*****************************************************/
/*          This java file is a part of the          */
/*                                                   */
/*           -  Plouf's Java IRC Client  -           */
/*                                                   */
/*   Copyright (C)  2002 - 2003 Philippe Detournay   */
/*                                                   */
/*         All contacts : theplouf@yahoo.com         */
/*                                                   */
/*  PJIRC is free software; you can redistribute     */
/*  it and/or modify it under the terms of the GNU   */
/*  General Public License as published by the       */
/*  Free Software Foundation; version 2 or later of  */
/*  the License.                                     */
/*                                                   */
/*  PJIRC is distributed in the hope that it will    */
/*  be useful, but WITHOUT ANY WARRANTY; without     */
/*  even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  General Public License for more details.         */
/*                                                   */
/*  You should have received a copy of the GNU       */
/*  General Public License along with PJIRC; if      */
/*  not, write to the Free Software Foundation,      */
/*  Inc., 59 Temple Place, Suite 330, Boston,        */
/*  MA  02111-1307  USA                              */
/*                                                   */
/*****************************************************/

package irc.dcc;

import irc.dcc.prv.*;
import java.io.*;
import java.util.*;
import irc.security.*;
import irc.*;

/**
 * The DCCFile, used for file transferts.
 */
public class DCCFile extends Source
{
  private OutputStream _os;
  private InputStream _is;
  private File _file;
  private ListenerGroup _listeners;
  private boolean _down=false;
  private int _size;
  private int _count;
  private DCCFileHandler _handler;

  /**
   * Create a new DCCFile.
   * @param f the file to transfert.
   * @param handler the file handler.
   */
  public DCCFile(IRCConfiguration config,File f,DCCFileHandler handler)
  {
    super(config,handler);
    _listeners=new ListenerGroup();
    _handler=handler;
    _count=0;
    _file=f;
  }

  /**
   * Add a listener.
   * @param lis listener to add.
   */
  public void addDCCFileListener(DCCFileListener lis)
  {
    _listeners.addListener(lis);
  }

  /**
   * Remove a listener.
   * @param lis listener to remove.
   */
  public void removeDCCFileListener(DCCFileListener lis)
  {
    _listeners.removeListener(lis);
  }

  /**
   * Prepare to send the file.
   */
  public void prepareSend()
  {
    try
    {
      _size=_ircConfiguration.getSecurityProvider().getFileSize(_file);
   //   _size=(int)_file.length();
      _is=new BufferedInputStream(_ircConfiguration.getSecurityProvider().getFileInputStream(_file));
      _down=false;
    }
    catch(Exception e)
    {
    }
  }

  /**
   * Read the next byte while sending.
   * @return next byte, or -1 if end of file reached.
   */
  public byte readByte()
  {
    try
    {
      _count++;
      _listeners.sendEvent("transmitted",new Integer(_count));
      return (byte)_is.read();
    }
    catch(Exception e)
    {
      return -1;
    }
  }

  /**
   * Get the file size.
   * @return the file size, in byte.
   */
  public int getSize()
  {
    return _size;
  }

  /**
   * Return true if the transfert is an upload transfert.
   * @return true if uplading, false otherwise.
   */
  public boolean isUploading()
  {
    return !isDownloading();
  }

  /**
   * Return true if the transfert is a download transfert.
   * @return true if downloading, false otherwise.
   */
  public boolean isDownloading()
  {
    return _down;
  }

  /**
   * Notify this file the sending is terminated.
   */
  public void fileSent()
  {
    try
    {
      _listeners.sendEvent("finished");
      _is.close();
    }
    catch(Exception e)
    {
    }
  }

  /**
   * Notify this file the sending has failed.
   */
  public void fileSentFailed()
  {
    try
    {
      _listeners.sendEvent("failed");
      _is.close();
    }
    catch(Exception e)
    {
    }
  }

  /**
   * Prepare to receive file.
   * @param size the file size.
   */
  public void prepareReceive(int size)
  {
    _down=true;
    _size=size;
    try
    {
      _os=new BufferedOutputStream(_ircConfiguration.getSecurityProvider().getFileOutputStream(_file));
    }
    catch(Exception e)
    {
      _os=null;
    }
  }

  /**
   * Write a new byte in the destination file.
   * @param b the new received byte.
   */
  public void byteReceived(byte b)
  {
    try
    {
      _count++;
      _listeners.sendEvent("transmitted",new Integer(_count));
      _os.write(b);
    }
    catch(Exception e)
    {
    }
  }

  /**
   * Notify this dcc file the file reception is terminated.
   */
  public void fileReceived()
  {
    try
    {
      _listeners.sendEvent("finished");
      _os.close();
    }
    catch(Exception e)
    {
    }

  }

  /**
   * Notify this dcc file the file reception has failed.
   */
  public void fileReceiveFailed()
  {
    try
    {
      _listeners.sendEvent("failed");
      _os.close();
    }
    catch(Exception e)
    {
    }
  }

  public String getName()
  {
    return _file.getName();
  }

  public void leave()
  {
    _handler.close();
    _handler.leave();
  }

  public boolean talkable()
  {
    return false;
  }

  public String getType()
  {
    return "DCCFile";
  }
}
