/*****************************************************/
/*          This java file is a part of the          */
/*                                                   */
/*           -  Plouf's Java IRC Client  -           */
/*                                                   */
/*   Copyright (C)  2002 - 2003 Philippe Detournay   */
/*                                                   */
/*         All contacts : theplouf@yahoo.com         */
/*                                                   */
/*  PJIRC is free software; you can redistribute     */
/*  it and/or modify it under the terms of the GNU   */
/*  General Public License as published by the       */
/*  Free Software Foundation; version 2 or later of  */
/*  the License.                                     */
/*                                                   */
/*  PJIRC is distributed in the hope that it will    */
/*  be useful, but WITHOUT ANY WARRANTY; without     */
/*  even the implied warranty of MERCHANTABILITY or  */
/*  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   */
/*  General Public License for more details.         */
/*                                                   */
/*  You should have received a copy of the GNU       */
/*  General Public License along with PJIRC; if      */
/*  not, write to the Free Software Foundation,      */
/*  Inc., 59 Temple Place, Suite 330, Boston,        */
/*  MA  02111-1307  USA                              */
/*                                                   */
/*****************************************************/

package irc.security;

import java.io.*;
import java.net.*;
import java.awt.*;

/**
 * The security provider. Provides an interface for accessing protected data or operation.
 */
public class SecurityProvider
{

  private SecuredProvider _provider;

  //private static final SecurityProvider _securityProvider=new SecurityProvider();

  /**
   * Create a new SecurityProvider.
   */
  public SecurityProvider()
  {
    if(tryProvider("MS")) return;
    if(tryProvider("SUN")) return;
    _provider=new DefaultSecuredProvider();
  }

  private boolean tryProvider(String name)
  {
    SecuredProvider old=_provider;
    try
    {
      Class cl=Class.forName("irc.security.prv.Specific"+name+"SecuredProvider");
      _provider=(SecuredProvider)cl.newInstance();
      if(!_provider.tryProvider()) throw new Exception();
      return true;
    }
    catch(Exception ex)
    {
      _provider=old;
      return false;
    }
  }

  /**
   * Get the used security provider.
   * @return the security provider to use.
   */
  /*public static final SecurityProvider getSecurityProvider()
  {
    return _securityProvider;
  }*/

  /**
   * Get the used provider name.
   * @return used provider name.
   */
  public String getProviderName()
  {
    return _provider.getName();
  }

  /**
   * Get a new Socket.
   * @param host server host.
   * @param port server port.
   * @throws UnknownHostException is host is not found.
   * @throws IOException if an error occurs.
   * @return a new Socket.
   */
  public Socket getSocket(String host,int port) throws UnknownHostException,IOException
  {
    return _provider.getSocket(host,port);
  }

  /**
   * Get a new ServerSocket.
   * @param port local port.
   * @throws IOException if an error occurs.
   */
  public ServerSocket getServerSocket(int port) throws IOException
  {
    return _provider.getServerSocket(port);
  }

  /**
   * Get an FileInputStream from a local file.
   * @param file the local file.
   * @throws IOException if an error occurs.
   * @return an FileInputStream from the file.
   */
  public FileInputStream getFileInputStream(File file) throws IOException
  {
    return _provider.getFileInputStream(file);
  }

  /**
   * Get an FileOutputStream to a local file.
   * @param file the local file.
   * @throws IOException if an error occurs.
   * @return an FileOutputStream from the file.
   */
  public FileOutputStream getFileOutputStream(File file) throws IOException
  {
    return _provider.getFileOutputStream(file);
  }

  /**
   * Get the file size.
   * @param file the file to get size.
   * @return the file size, of a negative value if not able.
   */
  public int getFileSize(File file)
  {
    return _provider.getFileSize(file);
  }

  /**
   * Open a load file dialog with the given title and return the user choice.
   * @param title dialog title.
   * @return user choice.
   */
  public File getLoadFile(String title)
  {
    return _provider.getLoadFile(title);
  }

  /**
   * Open a save file dialog with the given title and return the user choice.
   * @param title dialog title.
   * @return user choice.
   */
  public File getSaveFile(String title)
  {
    return _provider.getSaveFile(title);
  }

  /**
   * Get the local host address.
   * @return local host address.
   * @throws UnknownHostException if error occurs.
   */
  public InetAddress getLocalHost() throws UnknownHostException
  {
    return _provider.getLocalHost();
  }

  /**
   * Perform a dns resolve of the given address.
   * @param addr address to resolve.
   * @return resolved address.
   */
  public String resolve(InetAddress addr)
  {
    return _provider.resolve(addr);
  }


}

